@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import datetime
 import importlib
 import json
@@ -16,12 +14,10 @@ import shortuuid
 import vcode as mod_vcode
 from CodeConvert import CodeConvert as cc
 from redis import StrictRedis
-from redis._compat import iteritems, xrange
 from redis.client import bool_ok
 from redis.exceptions import ResponseError, WatchError
 from TimeConvert import TimeConvert as tc
 
-from .compat import basestring, bytes
 from .expires import BaseRedisExpires
 
 
@@ -78,7 +74,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
     def __str(self, x):
         if isinstance(x, int):
             return str(x)
-        return x if isinstance(x, basestring) else bytes(x)
+        return x if isinstance(x, (str, bytes)) else bytes(x)
 
     def __local_ymd(self, format='%Y-%m-%d'):
         return tc.local_string(format=format)
@@ -371,7 +367,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         Remove and return multi random member of set ``name``.
         """
         p = self.pipeline()
-        for _ in xrange(num):
+        for _ in range(num):
             p.spop(name)
         eles = p.execute()
         return eles, sum(x is not None for x in eles)
@@ -381,8 +377,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         # Srandmember isn't random enough
         # When number is close to ``the number of elements inside the set``
         memebers = self.srandmember(name, number=number)
-        if memebers:
-            random.shuffle(memebers)
+        random.shuffle(memebers)
         return memebers
 
     # ZSorts(Sorted Sets) Section
@@ -567,7 +562,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
     def hgetall_int(self, name, default=0):
         kvs = self.hgetall(name)
-        return {k: int(v or default) for (k, v) in iteritems(kvs)}
+        return {k: int(v or default) for (k, v) in kvs.items()}
 
     # FLOAT Section
     def get_float(self, name, default=0):
@@ -586,7 +581,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
     def hgetall_float(self, name, default=0):
         kvs = self.hgetall(name)
-        return {k: float(v or default) for (k, v) in iteritems(kvs)}
+        return {k: float(v or default) for (k, v) in kvs.items()}
 
     # STR Section
     def get_str(self, name, default=''):
@@ -605,10 +600,18 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
     def hgetall_str(self, name, default=''):
         kvs = self.hgetall(name)
-        return {k: (v or default) for (k, v) in iteritems(kvs)}
+        return {k: (v or default) for (k, v) in kvs.items()}
 
     # JSON Section
-    def set_json(self, name, value, ex=None, px=None, nx=False, xx=False, cls=None):
+    def __json_params(self, cls=None, json_params=None):
+        # TODO: Change to merge dict by {**a, **b} when drop support Python2.x
+        # cls for backwards compatibility
+        # return {**{'cls': cls}, **(json_params or {})}
+        json_params = json_params or {}
+        json_params['cls'] = cls
+        return json_params
+
+    def set_json(self, name, value, ex=None, px=None, nx=False, xx=False, cls=None, json_params=None):
         """
         Set the value at key ``name`` to ``json dumps value``.
 
@@ -620,33 +623,33 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
         ``xx`` if set to True, set the value at key ``name`` to ``value`` if it already exists.
         """
-        return self.set(name, json.dumps(value, cls=cls), ex=ex, px=px, nx=nx, xx=xx)
+        return self.set(name, json.dumps(value, **self.__json_params(cls, json_params)), ex=ex, px=px, nx=nx, xx=xx)
 
-    def setex_json(self, name, time, value, cls=None):
+    def setex_json(self, name, time, value, cls=None, json_params=None):
         """
         Set the value of key ``name`` to ``json dumps value`` that expires in ``time`` seconds.
 
         ``time`` can be represented by an integer or a Python timedelta object.
         """
-        return self.setex(name, time, json.dumps(value, cls=cls))
+        return self.setex(name, time, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def setnx_json(self, name, value, cls=None):
+    def setnx_json(self, name, value, cls=None, json_params=None):
         """
         Set the value of key ``name`` to ``json dumps value`` if key doesn't exist.
         """
-        return self.setnx(name, json.dumps(value, cls=cls))
+        return self.setnx(name, json.dumps(value, **self.__json_params(cls, json_params)))
 
     def get_json(self, name, default='{}'):
         return json.loads(self.get(name) or default)
 
-    def hset_json(self, name, key, value, cls=None):
-        return self.hset(name, key, json.dumps(value, cls=cls))
+    def hset_json(self, name, key, value, cls=None, json_params=None):
+        return self.hset(name, key, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def hsetnx_json(self, name, key, value, cls=None):
-        return self.hsetnx(name, key, json.dumps(value, cls=cls))
+    def hsetnx_json(self, name, key, value, cls=None, json_params=None):
+        return self.hsetnx(name, key, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def hmset_json(self, name, mapping, cls=None):
-        mapping = {k: json.dumps(v, cls=cls) for (k, v) in iteritems(mapping)}
+    def hmset_json(self, name, mapping, cls=None, json_params=None):
+        mapping = {k: json.dumps(v, **self.__json_params(cls, json_params)) for (k, v) in mapping.items()}
         return self.hmset(name, mapping)
 
     def hget_json(self, name, key, default='{}'):
@@ -662,25 +665,25 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
     def hgetall_json(self, name, default='{}'):
         kvs = self.hgetall(name)
-        return {k: json.loads(v or default) for (k, v) in iteritems(kvs)}
+        return {k: json.loads(v or default) for (k, v) in kvs.items()}
 
-    def lpush_json(self, name, value, cls=None):
-        return self.lpush(name, json.dumps(value, cls=cls))
+    def lpush_json(self, name, value, cls=None, json_params=None):
+        return self.lpush(name, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def rpush_json(self, name, value, cls=None):
-        return self.rpush(name, json.dumps(value, cls=cls))
+    def rpush_json(self, name, value, cls=None, json_params=None):
+        return self.rpush(name, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def lpushx_json(self, name, value, cls=None):
-        return self.lpushx(name, json.dumps(value, cls=cls))
+    def lpushx_json(self, name, value, cls=None, json_params=None):
+        return self.lpushx(name, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def rpushx_json(self, name, value, cls=None):
-        return self.rpushx(name, json.dumps(value, cls=cls))
+    def rpushx_json(self, name, value, cls=None, json_params=None):
+        return self.rpushx(name, json.dumps(value, **self.__json_params(cls, json_params)))
 
-    def lpushnx_json(self, name, value, cls=None, force=True):
-        return self.lpushnx(name, json.dumps(value, cls=cls), force=force)
+    def lpushnx_json(self, name, value, cls=None, force=True, json_params=None):
+        return self.lpushnx(name, json.dumps(value, **self.__json_params(cls, json_params)), force=force)
 
-    def rpushnx_json(self, name, value, cls=None, force=True):
-        return self.rpushnx(name, json.dumps(value, cls=cls), force=force)
+    def rpushnx_json(self, name, value, cls=None, force=True, json_params=None):
+        return self.rpushnx(name, json.dumps(value, **self.__json_params(cls, json_params)), force=force)
 
     def lpop_json(self, name, default='{}'):
         return json.loads(self.lpop(name) or default)
@@ -688,13 +691,13 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
     def rpop_json(self, name, default='{}'):
         return json.loads(self.rpop(name) or default)
 
-    def blpop_json(self, keys, timeout=0, cls=None):
+    def blpop_json(self, keys, timeout=0, cls=None, json_params=None):
         kv = self.blpop(keys, timeout=timeout)
-        return (kv[0], json.loads(kv[1], cls=cls)) if kv else (None, None)
+        return (kv[0], json.loads(kv[1], **self.__json_params(cls, json_params))) if kv else (None, None)
 
-    def brpop_json(self, keys, timeout=0, cls=None):
+    def brpop_json(self, keys, timeout=0, cls=None, json_params=None):
         kv = self.brpop(keys, timeout=timeout)
-        return (kv[0], json.loads(kv[1], cls=cls)) if kv else (None, None)
+        return (kv[0], json.loads(kv[1], **self.__json_params(cls, json_params))) if kv else (None, None)
 
     # Locks Section
     def __lock_key(self, name):
@@ -1018,7 +1021,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
     def gvcode_add(self, num=10):
         if num <= 0:
             raise ValueError('The num argument should be positive')
-        gvcodes = (self.__gvcode_str() for _ in xrange(num))
+        gvcodes = (self.__gvcode_str() for _ in range(num))
         return self.sadd(self._gvcode_key(), *gvcodes)
 
     def gvcode_initial(self, num=10):
@@ -1137,14 +1140,14 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
         ``release_lock_when_error`` indicates whether release lock when error or not.
         """
-        callbacks = {k: self.__callable_func(v) for k, v in iteritems(callbacks)}
-        callbacks = {k: v for k, v in iteritems(callbacks) if v}
+        callbacks = {k: self.__callable_func(v) for k, v in callbacks.items()}
+        callbacks = {k: v for k, v in callbacks.items() if v}
 
         final_process_lock_key = 'process:lock:{0}'.format(process_lock_key or delayed)
         final_logger = delayed_logger or logger
 
         final_logger.info('>>> Available callbacks ({0}):'.format(len(callbacks)))
-        for k, v in iteritems(callbacks):
+        for k, v in callbacks.items():
             final_logger.info('  * {0}: {1}'.format(k, v))
 
         self.__release_lock_when_launch(release_lock_when_launch, release_lock_eth0_inet_addr, final_process_lock_key, delayed, final_logger)
